@@ -1,14 +1,23 @@
-broyden <- function(Ffun, x0, maxiter = 20, tol = .Machine$double.eps^(1/2)) {
+broyden <- function(Ffun, x0, J0 = NULL, ...,
+                    maxiter = 100, tol = .Machine$double.eps^(1/2)) {
     if (!is.numeric(x0))
         stop("Argument 'x0' must be a numeric (row or column) vector.")
-    F <- match.fun(Ffun)
+    fun <- match.fun(Ffun)
+    F <- function(x) fun(x, ...)
     y0 <- F(x0)
     if (length(x0) != length(y0))
         stop("Function 'F' must be 'square', i.e. from R^n to R^n .")
 
     # Compute once the Jacobian and its inverse
-    A0 <- jacobian(F, x0)
+    if (is.null(J0)) {
+        A0 <- jacobian(F, x0)
+    } else {
+        A0 <- J0
+    }
+
     B0 <- inv(A0)
+    if (any(is.infinite(B0)))
+        B0 <- diag(length(x0))
 
     # Secant-like step in Broyden's method
     xnew <- x0 - B0 %*% y0
@@ -36,5 +45,5 @@ broyden <- function(Ffun, x0, maxiter = 20, tol = .Machine$double.eps^(1/2)) {
         warning(paste("Not converged: Max number of iterations reached."))
 
     fnew <- sqrt(sum(ynew^2))
-    return(list(zero = xnew, fnorm = fnew, niter = k))
+    return(list(zero = c(xnew), fnorm = fnew, niter = k))
 }
